// Addressdefinitions for H8S/2329
// Johann Gysin
// 17.10.2003

#ifndef _ioh82329_h
#define _ioh82329_h

// Bitdefinitions
#define B0  1
#define B1  2
#define B2  4
#define B3  8
#define B4  16
#define B5  32
#define B6  64
#define B7  128
#define B8  256
#define B9  512
#define B10 1024
#define B11 2048
#define B12 4096
#define B13 8192
#define B14 16384
#define B15 32768

// Divers Registers
#define SYSCR   (*(volatile unsigned char*)(0xFFFFFF39)) // System Control Register
#define SYSCR2  (*(volatile unsigned char*)(0xFFFFFF42)) // System Control Register 2
#define MDCR    (*(volatile unsigned char*)(0xFFFFFF3B)) // Mode Control Register

// Power Down Mode
#define SBYCR   (*(volatile unsigned char*)(0xFFFFFF38)) // Standby Control Register
#define SCKCR   (*(volatile unsigned char*)(0xFFFFFF3A)) // System Clock Control Register
#define MSTPCR  (*(volatile unsigned int*)(0xFFFFFF3C))  // Module Stop Control Register

// Ports
#define PFCR1   (*(volatile unsigned char*)(0xFFFFFF45)) // Port Function Control Register 1
#define PFCR2   (*(volatile unsigned char*)(0xFFFFFFAC)) // Port Function Control Register 2

// Port 1
#define P1DDR   (*(volatile unsigned char*)(0xFFFFFEB0)) // Port 1 Data Direction Register
#define P1DR    (*(volatile unsigned char*)(0xFFFFFF60)) // Port 1 Data Register
#define PORT1   (*(volatile unsigned char*)(0xFFFFFF50)) // Port 1 Register

// Port 2
#define P2DDR   (*(volatile unsigned char*)(0xFFFFFEB1)) // Port 2 Data Direction Register
#define P2DR    (*(volatile unsigned char*)(0xFFFFFF61)) // Port 2 Data Register
#define PORT2   (*(volatile unsigned char*)(0xFFFFFF51)) // Port 2 Register

// Port 3
#define P3DDR   (*(volatile unsigned char*)(0xFFFFFEB2)) // Port 3 Data Direction Register
#define P3DR    (*(volatile unsigned char*)(0xFFFFFF62)) // Port 3 Data Register
#define PORT3   (*(volatile unsigned char*)(0xFFFFFF52)) // Port 3 Register
#define P3ODR   (*(volatile unsigned char*)(0xFFFFFE76)) // Port 3 Open-Drain Control Register

// Port 4
#define PORT4   (*(volatile unsigned char*)(0xFFFFFF53)) // Port 4 Register

// Port 5
#define P5DDR   (*(volatile unsigned char*)(0xFFFFFEB4)) // Port 5 Data Direction Register
#define P5DR    (*(volatile unsigned char*)(0xFFFFFF54)) // Port 5 Data Register
#define PORT5   (*(volatile unsigned char*)(0xFFFFFF52)) // Port 5 Register

// Port 6
#define P6DDR   (*(volatile unsigned char*)(0xFFFFFEB5)) // Port 6 Data Direction Register
#define P6DR    (*(volatile unsigned char*)(0xFFFFFF65)) // Port 6 Data Register
#define PORT6   (*(volatile unsigned char*)(0xFFFFFF55)) // Port 6 Register

// Port A
#define PADDR   (*(volatile unsigned char*)(0xFFFFFEB9)) // Port A Data Direction Register
#define PADR    (*(volatile unsigned char*)(0xFFFFFF69)) // Port A Data Register
#define PORTA   (*(volatile unsigned char*)(0xFFFFFF59)) // Port A Register
#define PAPCR   (*(volatile unsigned char*)(0xFFFFFF70)) // Port A MOS Pull-Up Control Register
#define PAODR   (*(volatile unsigned char*)(0xFFFFFF77)) // Port A Open-Drain Control Register

// Port B
#define PBDDR   (*(volatile unsigned char*)(0xFFFFFEBA)) // Port B Data Direction Register
#define PBDR    (*(volatile unsigned char*)(0xFFFFFF6A)) // Port B Data Register
#define PORTB   (*(volatile unsigned char*)(0xFFFFFF5A)) // Port B Register
#define PBPCR   (*(volatile unsigned char*)(0xFFFFFF71)) // Port B MOS Pull-Up Control Register

// Port C
#define PCDDR   (*(volatile unsigned char*)(0xFFFFFEBB)) // Port C Data Direction Register
#define PCDR    (*(volatile unsigned char*)(0xFFFFFF6B)) // Port C Data Register
#define PORTC   (*(volatile unsigned char*)(0xFFFFFF5B)) // Port C Register
#define PCPCR   (*(volatile unsigned char*)(0xFFFFFF72)) // Port C MOS Pull-Up Control Register

// Port D
#define PDDDR   (*(volatile unsigned char*)(0xFFFFFEBC)) // Port D Data Direction Register
#define PDDR    (*(volatile unsigned char*)(0xFFFFFF6C)) // Port D Data Register
#define PORTD   (*(volatile unsigned char*)(0xFFFFFF5C)) // Port D Register
#define PDPCR   (*(volatile unsigned char*)(0xFFFFFF73)) // Port D MOS Pull-Up Control Register

// Port E
#define PEDDR   (*(volatile unsigned char*)(0xFFFFFEBD)) // Port E Data Direction Register
#define PEDR    (*(volatile unsigned char*)(0xFFFFFF6D)) // Port E Data Register
#define PORTE   (*(volatile unsigned char*)(0xFFFFFF5D)) // Port E Register
#define PEPCR   (*(volatile unsigned char*)(0xFFFFFF74)) // Port E MOS Pull-Up Control Register

// Port F
#define PFDDR   (*(volatile unsigned char*)(0xFFFFFEBE)) // Port F Data Direction Register
#define PFDR    (*(volatile unsigned char*)(0xFFFFFF6E)) // Port F Data Register
#define PORTF   (*(volatile unsigned char*)(0xFFFFFF5E)) // Port F Register

// Port G
#define PGDDR   (*(volatile unsigned char*)(0xFFFFFEBF)) // Port G Data Direction Register
#define PGDR    (*(volatile unsigned char*)(0xFFFFFF6F)) // Port G Data Register
#define PORTG   (*(volatile unsigned char*)(0xFFFFFF5F)) // Port G Register

// D/A
#define DADR0   (*(volatile unsigned char*)(0xFFFFFFA4)) // D/A Data Register 0
#define DADR1   (*(volatile unsigned char*)(0xFFFFFFA5)) // D/A Data Register 1
#define DACR01  (*(volatile unsigned char*)(0xFFFFFFA6)) // D/A Control Register 1

// SCI0
#define SMR0    (*(volatile unsigned char*)(0xFFFFFF78)) // Serial Mode Register 0
#define BRR0    (*(volatile unsigned char*)(0xFFFFFF79)) // Bit Rate Register 0
#define SCR0    (*(volatile unsigned char*)(0xFFFFFF7A)) // Serial Control Register 0
#define TDR0    (*(volatile unsigned char*)(0xFFFFFF7B)) // Transmit Data Register 0
#define SSR0    (*(volatile unsigned char*)(0xFFFFFF7C)) // Serial Status Register 0
#define RDR0    (*(volatile unsigned char*)(0xFFFFFF7D)) // Receive Data Register 0
#define SCMR0   (*(volatile unsigned char*)(0xFFFFFF7E)) // Smart Card Mode Register 0

// SCI1
#define SMR1    (*(volatile unsigned char*)(0xFFFFFF80)) // Serial Mode Register 1
#define BRR1    (*(volatile unsigned char*)(0xFFFFFF81)) // Bit Rate Register 1
#define SCR1    (*(volatile unsigned char*)(0xFFFFFF82)) // Serial Control Register 1
#define TDR1    (*(volatile unsigned char*)(0xFFFFFF83)) // Transmit Data Register 1
#define SSR1    (*(volatile unsigned char*)(0xFFFFFF84)) // Serial Status Register 1
#define RDR1    (*(volatile unsigned char*)(0xFFFFFF85)) // Receive Data Register 1
#define SCMR1   (*(volatile unsigned char*)(0xFFFFFF86)) // Smart Card Mode Register 1

// SCI2
#define SMR2    (*(volatile unsigned char*)(0xFFFFFF88)) // Serial Mode Register 2
#define BRR2    (*(volatile unsigned char*)(0xFFFFFF89)) // Bit Rate Register 2
#define SCR2    (*(volatile unsigned char*)(0xFFFFFF8A)) // Serial Control Register 2
#define TDR2    (*(volatile unsigned char*)(0xFFFFFF8B)) // Transmit Data Register 2
#define SSR2    (*(volatile unsigned char*)(0xFFFFFF8C)) // Serial Status Register 2
#define RDR2    (*(volatile unsigned char*)(0xFFFFFF8D)) // Receive Data Register 2
#define SCMR2   (*(volatile unsigned char*)(0xFFFFFF8E)) // Smart Card Mode Register 2

// TPU0
#define TCR0    (*(volatile unsigned char*)(0xFFFFFFD0)) // Timer Control Register 0
#define TMDR0   (*(volatile unsigned char*)(0xFFFFFFD1)) // Timer Mode Register 0
#define TIOR0   (*(volatile unsigned int*)(0xFFFFFFD2))  // Timer I/O Control Register 0
#define TIER0   (*(volatile unsigned char*)(0xFFFFFFD4)) // Timer Interrupt Enable Register 0
#define TSR0    (*(volatile unsigned char*)(0xFFFFFFD5)) // Timer Status Register 0
#define TCNT0   (*(volatile unsigned int*)(0xFFFFFFD6))  // Timer Counter 0
#define TGR0A   (*(volatile unsigned char*)(0xFFFFFFD8)) // Timer General Register 0A
#define TGR0B   (*(volatile unsigned char*)(0xFFFFFFDA)) // Timer General Register 0B
#define TGR0C   (*(volatile unsigned char*)(0xFFFFFFDC)) // Timer General Register 0C
#define TGR0D   (*(volatile unsigned char*)(0xFFFFFFDE)) // Timer General Register 0D

// TPU1
#define TCR1    (*(volatile unsigned char*)(0xFFFFFFE0)) // Timer Control Register 1
#define TMDR1   (*(volatile unsigned char*)(0xFFFFFFE1)) // Timer Mode Register 1
#define TIOR1   (*(volatile unsigned int*)(0xFFFFFFE2))  // Timer I/O Control Register 1
#define TIER1   (*(volatile unsigned char*)(0xFFFFFFE4)) // Timer Interrupt Enable Register 1
#define TSR1    (*(volatile unsigned char*)(0xFFFFFFE5)) // Timer Status Register 1
#define TCNT1   (*(volatile unsigned int*)(0xFFFFFFE6))  // Timer Counter 1
#define TGR1A   (*(volatile unsigned char*)(0xFFFFFFE8)) // Timer General Register 1A
#define TGR1B   (*(volatile unsigned char*)(0xFFFFFFEA)) // Timer General Register 1B

// TPU2
#define TCR2    (*(volatile unsigned char*)(0xFFFFFFF0)) // Timer Control Register 2
#define TMDR2   (*(volatile unsigned char*)(0xFFFFFFF1)) // Timer Mode Register 2
#define TIOR2   (*(volatile unsigned int*)(0xFFFFFFF2))  // Timer I/O Control Register 2
#define TIER2   (*(volatile unsigned char*)(0xFFFFFFF4)) // Timer Interrupt Enable Register 2
#define TSR2    (*(volatile unsigned char*)(0xFFFFFFF5)) // Timer Status Register 2
#define TCNT2   (*(volatile unsigned int*)(0xFFFFFFF6))  // Timer Counter 2
#define TGR2A   (*(volatile unsigned char*)(0xFFFFFFF8)) // Timer General Register 2A
#define TGR2B   (*(volatile unsigned char*)(0xFFFFFFFA)) // Timer General Register 2B

// TPU3
#define TCR3    (*(volatile unsigned char*)(0xFFFFFE80)) // Timer Control Register 3
#define TMDR3   (*(volatile unsigned char*)(0xFFFFFE81)) // Timer Mode Register 3
#define TIOR3   (*(volatile unsigned int*)(0xFFFFFE82))  // Timer I/O Control Register 3
#define TIER3   (*(volatile unsigned char*)(0xFFFFFE84)) // Timer Interrupt Enable Register 3
#define TSR3    (*(volatile unsigned char*)(0xFFFFFE85)) // Timer Status Register 3
#define TCNT3   (*(volatile unsigned int*)(0xFFFFFE86))  // Timer Counter 3
#define TGR3A   (*(volatile unsigned char*)(0xFFFFFE88)) // Timer General Register 3A
#define TGR3B   (*(volatile unsigned char*)(0xFFFFFE8A)) // Timer General Register 3B
#define TGR3C   (*(volatile unsigned char*)(0xFFFFFE8C)) // Timer General Register 3C
#define TGR3D   (*(volatile unsigned char*)(0xFFFFFE8E)) // Timer General Register 3D

// TPU4
#define TCR4    (*(volatile unsigned char*)(0xFFFFFE90)) // Timer Control Register 4
#define TMDR4   (*(volatile unsigned char*)(0xFFFFFE91)) // Timer Mode Register 4
#define TIOR4   (*(volatile unsigned int*)(0xFFFFFE92))  // Timer I/O Control Register 4
#define TIER4   (*(volatile unsigned char*)(0xFFFFFE94)) // Timer Interrupt Enable Register 4
#define TSR4    (*(volatile unsigned char*)(0xFFFFFE95)) // Timer Status Register 4
#define TCNT4   (*(volatile unsigned int*)(0xFFFFFE96))  // Timer Counter 4
#define TGR4A   (*(volatile unsigned char*)(0xFFFFFE98)) // Timer General Register 4A
#define TGR4B   (*(volatile unsigned char*)(0xFFFFFE9A)) // Timer General Register 4B

// TPU5
#define TCR5    (*(volatile unsigned char*)(0xFFFFFEA0)) // Timer Control Register 5
#define TMDR5   (*(volatile unsigned char*)(0xFFFFFEA1)) // Timer Mode Register 5
#define TIOR5   (*(volatile unsigned int*)(0xFFFFFEA2))  // Timer I/O Control Register 5
#define TIER5   (*(volatile unsigned char*)(0xFFFFFEA4)) // Timer Interrupt Enable Register 5
#define TSR5    (*(volatile unsigned char*)(0xFFFFFEA5)) // Timer Status Register 5
#define TCNT5   (*(volatile unsigned int*)(0xFFFFFEA6))  // Timer Counter 5
#define TGR5A   (*(volatile unsigned char*)(0xFFFFFEA8)) // Timer General Register 5A
#define TGR5B   (*(volatile unsigned char*)(0xFFFFFEAA)) // Timer General Register 5B

// All TPU Channels
#define TSTR    (*(volatile unsigned char*)(0xFFFFFFC0)) // Timer Start Register
#define TSYR    (*(volatile unsigned char*)(0xFFFFFFC1)) // Timer Syncro Register

// PPG
#define PCR     (*(volatile unsigned char*)(0xFFFFFF46)) // PPG Output Control Register
#define PMR     (*(volatile unsigned char*)(0xFFFFFF47)) // PPG Output Mode Register
#define NDER    (*(volatile unsigned int*)(0xFFFFFF48))  // Next Data Enable Register
#define PODR    (*(volatile unsigned int*)(0xFFFFFF4A))  // Output Data Register
#define NDRa    (*(volatile unsigned int*)(0xFFFFFF4C))  // Next Data Register
#define NDRb    (*(volatile unsigned int*)(0xFFFFFF4E))  // Next Data Register

// 8-Bit Timer 0
#define _TCR0   (*(volatile unsigned char*)(0xFFFFFFB0)) // Timer Control Register 0
#define _TCSR0  (*(volatile unsigned char*)(0xFFFFFFB2)) // Timer Control/Status Register 0
#define _TCORA0 (*(volatile unsigned char*)(0xFFFFFFB4)) // Timer Constant Register A0
#define _TCORB0 (*(volatile unsigned char*)(0xFFFFFFB6)) // Timer Constant Register B0
#define _TCNT0  (*(volatile unsigned char*)(0xFFFFFFB8)) // Timer Counter 0

// 8-Bit Timer 1
#define _TCR1   (*(volatile unsigned char*)(0xFFFFFFB1)) // Timer Control Register 1
#define _TCSR1  (*(volatile unsigned char*)(0xFFFFFFB3)) // Timer Control/Status Register 1
#define _TCORA1 (*(volatile unsigned char*)(0xFFFFFFB5)) // Timer Constant Register A1
#define _TCORB1 (*(volatile unsigned char*)(0xFFFFFFB7)) // Timer Constant Register B1
#define _TCNT1  (*(volatile unsigned char*)(0xFFFFFFB9)) // Timer Counter 1

// WDT
#define WDTCSRw (*(volatile unsigned char*)(0xFFFFFFBC)) // Timer Control/Status Register (Write)
#define WDTCSRr (*(volatile unsigned char*)(0xFFFFFFBC)) // Timer Control/Status Register (Read)
#define WDTCNTw (*(volatile unsigned char*)(0xFFFFFFBC)) // Timer Counter (Write)
#define WDTCNTr (*(volatile unsigned char*)(0xFFFFFFBD)) // Timer Counter (Read)
#define RSTCSRw (*(volatile unsigned char*)(0xFFFFFFBE)) // Reset Control/Status Register (Write)
#define RSTCSRr (*(volatile unsigned char*)(0xFFFFFFBF)) // Reset Control/Status Register (Read)

// Interrupt Controller Registers
#define ISCRH   (*(volatile unsigned char*)(0xFFFFFF2C)) // IRQ Sense Control Register H
#define ISCRL   (*(volatile unsigned char*)(0xFFFFFE2D)) // IRQ Sense Control Register L
#define IER     (*(volatile unsigned char*)(0xFFFFFE2E)) // IRQ Enable Register
#define ISR     (*(volatile unsigned char*)(0xFFFFFE2F)) // IRQ Status Register
#define IPRA    (*(volatile unsigned char*)(0xFFFFFEC4)) // Interrupt Priority Register A
#define IPRB    (*(volatile unsigned char*)(0xFFFFFEC5)) // Interrupt Priority Register B
#define IPRC    (*(volatile unsigned char*)(0xFFFFFEC6)) // Interrupt Priority Register C
#define IPRD    (*(volatile unsigned char*)(0xFFFFFEC7)) // Interrupt Priority Register D
#define IPRE    (*(volatile unsigned char*)(0xFFFFFEC8)) // Interrupt Priority Register E
#define IPRF    (*(volatile unsigned char*)(0xFFFFFEC9)) // Interrupt Priority Register F
#define IPRG    (*(volatile unsigned char*)(0xFFFFFECA)) // Interrupt Priority Register G
#define IPRH    (*(volatile unsigned char*)(0xFFFFFECB)) // Interrupt Priority Register H
#define IPRI    (*(volatile unsigned char*)(0xFFFFFECC)) // Interrupt Priority Register I
#define IPRJ    (*(volatile unsigned char*)(0xFFFFFECD)) // Interrupt Priority Register J
#define IPRK    (*(volatile unsigned char*)(0xFFFFFECE)) // Interrupt Priority Register K

// Bus Controller
#define ABWCR   (*(volatile unsigned char*)(0xFFFFFED0)) // Bus Width Control Register
#define ASTCR   (*(volatile unsigned char*)(0xFFFFFED1)) // Access State Control Register
#define WCRH    (*(volatile unsigned char*)(0xFFFFFED2)) // Wait Control Register H
#define WCRL    (*(volatile unsigned char*)(0xFFFFFED3)) // Wait Control Register L
#define BCRH    (*(volatile unsigned char*)(0xFFFFFED4)) // Bus Control Register H
#define BCRL    (*(volatile unsigned char*)(0xFFFFFED5)) // Bus Control Register L
#define MCR     (*(volatile unsigned char*)(0xFFFFFED6)) // Memory Control Register
#define DRAMCR  (*(volatile unsigned char*)(0xFFFFFED7)) // DRAM Control Register
#define RTCNT   (*(volatile unsigned char*)(0xFFFFFED8)) // Refresh Timer Counter
#define RTCOR   (*(volatile unsigned char*)(0xFFFFFED9)) // Refresh Time Constant Register

// DMAC0
#define MAR0A   (*(volatile unsigned char*)(0xFFFFFEE0)) // Memory Address Register 0A
#define IOAR0A  (*(volatile unsigned char*)(0xFFFFFEE4)) // I/O Address Register 0A
#define ETCR0A  (*(volatile unsigned char*)(0xFFFFFEE6)) // Transfer Count Register 0A
#define MAR0B   (*(volatile unsigned char*)(0xFFFFFEE8)) // Memory Address Register 0B
#define IOAR0B  (*(volatile unsigned char*)(0xFFFFFEEC)) // I/O Address Register 0B
#define ETCR0B  (*(volatile unsigned char*)(0xFFFFFEEE)) // Transfer Count Register 0B

// DMAC1
#define MAR1A   (*(volatile unsigned char*)(0xFFFFFEF0)) // Memory Address Register 1A
#define IOAR1A  (*(volatile unsigned char*)(0xFFFFFEF4)) // I/O Address Register 1A
#define ETCR1A  (*(volatile unsigned char*)(0xFFFFFEF6)) // Transfer Count Register 1A
#define MAR1B   (*(volatile unsigned char*)(0xFFFFFEF8)) // Memory Address Register 1B
#define IOAR1B  (*(volatile unsigned char*)(0xFFFFFEFC)) // I/O Address Register 1B
#define ETCR1B  (*(volatile unsigned char*)(0xFFFFFEFE)) // Transfer Count Register 1B

// Both DMAC Channels
#define DMAWER  (*(volatile unsigned char*)(0xFFFFFF00)) // DMA Write Enable Register
#define DMATCR  (*(volatile unsigned char*)(0xFFFFFF01)) // DMA Terminal Control Register
#define DMACR0A (*(volatile unsigned char*)(0xFFFFFF02)) // DMA Control Register 0A
#define DMACR0B (*(volatile unsigned char*)(0xFFFFFF03)) // DMA Control Register 0B
#define DMACR1A (*(volatile unsigned char*)(0xFFFFFF04)) // DMA Control Register 1A
#define DMACR1B (*(volatile unsigned char*)(0xFFFFFF05)) // DMA Control Register 1B
#define DMABCR  (*(volatile unsigned char*)(0xFFFFFF06)) // DMA Band Control Register

// ADC
#define ADDRA   (*(volatile unsigned int*)(0xFFFFFF90))  // A/D Data Register A
#define ADDRB   (*(volatile unsigned int*)(0xFFFFFF92))  // A/D Data Register B
#define ADDRC   (*(volatile unsigned int*)(0xFFFFFF94))  // A/D Data Register C
#define ADDRD   (*(volatile unsigned int*)(0xFFFFFF96))  // A/D Data Register D
#define ADCSR   (*(volatile unsigned char*)(0xFFFFFF98)) // A/D Control/Status Register
#define ADCR    (*(volatile unsigned char*)(0xFFFFFF99)) // A/D Control Register

#endif
