/* Linker Commands for H8S 2633 in Mode 7 (Single Chip)
 * Original from Johan Henning
 * Little modifications by Johann Gysin
 * 15.10.2003
 */
/*
 *  Linker command file for stand alone operation
 *
 *
 *  Memory Map:
 *
 *  H 000000    +-------------------+     \
 *              |     Vectors       |      |
 *  H 000000?   +-------------------+      |
 *              |                   |      |
 *  H 010000    +-------------------+      |- 328 KByte
 *              |                   |      |
 *              |      FLASH        |      |
 *              |                   |      |
 *  H 060000    +-------------------+     /
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *  H FF7C00    +-------------------+
 *              |                   |     \
 *              |   Internal RAM    |      |- 32 KByte
 *              |                   |     /
 *  H FFFBFF    +-------------------+
 *              |                   |
 *  H FFFE50    +-------------------+
 *              |                   |
 *              |Internal Registers |
 *              |                   |
 *  H FFFF07    +-------------------+
 *              |                   |
 *  H FFFF28    +-------------------+
 *              |Internal Registers |
 *  H FFFFFF    +-------------------+
 *
 */
SEARCH_DIR (C:\cygnus\lib\H8300s\)
OUTPUT_FORMAT("coff-h8300")
OUTPUT_ARCH(h8300s)

MEMORY
{
  rom   : o = 0x0000000, l = 328k       /* code space */
  ram   : o = 0x0ff7c00, l = 0x07FFF    /* data and stack space */
}

SECTIONS 
{
  .vectors :
    {
    _vect = .;
    *(.vect);
    *(.progid);
    *(.monitor);
    } > rom

  .tors :
    {
    ___ctors = . ;
    *(.ctors)
    ___ctors_end = . ;
    ___dtors = . ;
    *(.dtors)
    ___dtors_end = . ;
    } > rom

  .text :
    {
    *(.text)
    *(.rodata)
     _etext = ALIGN(16);
     } > rom

  .data : AT(_etext)    /* places a ROM image at _etext */
    {
    _data = .;
    *(.data)
    _edata = .; 
    } > ram

  .bss (NOLOAD) :
    {
    _bss = . ;
    *(.bss)
    *(COMMON)
    _ebss = .;
    _end = .;  
    } > ram

/* .uir stands for un-initialized-ram  */
  .uir :
    {
    _uir  = .;
    *(.uir)
    _euir = .;
    } > ram

  .stack (NOLOAD) :
    {
    _stack_last  = .;  /* deepest location of the stack                              */
    . += 0x080;        /* reserve 128 bytes stack space for Cstart.asm and main.cpp  */
    _stack = .; 
    _stack_first = .;  /* shallowest location of the stack                           */
    *(.stack)
    } > ram

  .stab 0 (NOLOAD) :
    {
    [ .stab ]
    }
  .stabstr 0 (NOLOAD) :
    {
    [ .stabstr ]
    }
}
